print("import: 'megaradrp'")
import megaradrp

print("import: 'megaradrp.core'")
import megaradrp.core

print("import: 'megaradrp.core.tests'")
import megaradrp.core.tests

print("import: 'megaradrp.instrument'")
import megaradrp.instrument

print("import: 'megaradrp.instrument.configs'")
import megaradrp.instrument.configs

print("import: 'megaradrp.processing'")
import megaradrp.processing

print("import: 'megaradrp.processing.tests'")
import megaradrp.processing.tests

print("import: 'megaradrp.products'")
import megaradrp.products

print("import: 'megaradrp.recipes'")
import megaradrp.recipes

print("import: 'megaradrp.recipes.auxiliary'")
import megaradrp.recipes.auxiliary

print("import: 'megaradrp.recipes.calibration'")
import megaradrp.recipes.calibration

print("import: 'megaradrp.recipes.calibration.tests'")
import megaradrp.recipes.calibration.tests

print("import: 'megaradrp.recipes.combined'")
import megaradrp.recipes.combined

print("import: 'megaradrp.recipes.scientific'")
import megaradrp.recipes.scientific

print("import: 'megaradrp.recipes.tests'")
import megaradrp.recipes.tests

print("import: 'megaradrp.simulation'")
import megaradrp.simulation

print("import: 'megaradrp.tests'")
import megaradrp.tests

